#!/usr/bin/perl -w
# $Id: Bkp.pm.in,v 1.10 2003/12/02 13:28:45 x99laine Exp $
# Copyright (c) 2003 Jeremy Lain

package Bkp;

use strict;
use Config::General;

# visible variables
our $confile = "/etc/bkp/bkp.conf";
our $debug;

# a valid filename
my $mask_file = "[a-zA-Z0-9_\\-\\. ]+";
# a valid full path
my $mask_fullpath = "\\/(?:$mask_file\\/)*($mask_file)?(?:\\/)?";
# a valid relative path
my $mask_relpath = "$mask_file(?:\\/$mask_file)*(?:\\/)?";
# a valid FTP url
my $mask_auth = "[^:@]+(:[^@]*)?"; 
my $mask_hostfrag = "[a-zA-Z0-9_\\-]+";
my $mask_host = "$mask_hostfrag(\\.$mask_hostfrag)*";
my $mask_url = "(ftp|fish):\\/\\/($mask_auth@)?($mask_host)(:[0-9]+)?($mask_fullpath)";

sub dprint {
  my $text = shift;
  print $text if ($debug);
}

sub read_val {
  my($default,$value) = @_;
  if ($value) { return $value; }
  else { return $default; }
}

sub read_conf {
  my $file = shift;
  my($config,%conf,%rawconf);
  
  dprint("Using $file as config file\n");
  if (!(-r $file)) {
    print "No valid configuration file found, aborting\n";
    exit(1);
  }
  
  $config = new Config::General( -file =>"$file",
                                 -AllowMultiOptions =>"yes",
                                 -LowerCaseNames =>"yes",);
  %rawconf = $config->getall;

  # import known values
  $conf{'savedir'} = &read_val('',$rawconf{'savedir'});
  @{$conf{'all'}{'databases'}} = split / /,$rawconf{'databases'};
  @{$conf{'all'}{'sections'}} = split / /,$rawconf{'sections'};
  @{$conf{'all'}{'remotes'}} = split / /,$rawconf{'remotes'};
  $conf{'maxdays'}{'databases'} = &read_val(15,$rawconf{'databases_maxdays'});
  $conf{'maxdays'}{'sections'} = &read_val(15,$rawconf{'sections_maxdays'});
  $conf{'maxretries'}{'remotes'} = &read_val(5,$rawconf{'remotes_maxretries'});

  # perform checks
  if (!$conf{'savedir'}) {
    print("no valid savedir, aborting\n");
    exit(1);
  }

  # read sections
  my %sections;
  foreach my $section (@{$conf{'all'}{'sections'}}) {
    my (@targets,$entry);
  
    if (!$rawconf{$section.'_targets'}) {
      print("could not find targets for section $section\n");
      exit(1);
    }
    
    $sections{$section}{'maxdays'} = &read_val($conf{'maxdays'}{'sections'},$rawconf{$section.'_maxdays'});
    $sections{$section}{'class'} = &read_val("",$rawconf{$section.'_class'});
    
    @targets = split / /,&read_val("", $rawconf{$section.'_targets'});
    foreach $entry(@targets) {
      if  ($entry !~ /^($mask_fullpath)\:($mask_relpath(?:,$mask_relpath)*)\:$mask_file(?:\:[^\:]*)?$/) {
        print("invalid entry $entry for section $section\n");
				exit(1);
      }
    }

		map s/,/ /g, @targets;
    @{$sections{$section}{'targets'}} = @targets;
  }
  %{$conf{'sections'}} = %sections;

  # read remote entries
  my %remotes;
  foreach my $remote (@{$conf{'all'}{'remotes'}}) {
    if ($rawconf{$remote.'_url'} !~ /^$mask_url$/) {
      print("invalid remote url for $remote\n");
      exit(1);
    }
    $remotes{$remote}{'url'} = &read_val("", $rawconf{$remote.'_url'});
    $remotes{$remote}{'class'} = &read_val("",$rawconf{$remote.'_class'});
    @{$remotes{$remote}{'sections'}} = split / /,read_val("", $rawconf{$remote.'_sections'});
  }
  %{$conf{'remotes'}} = %remotes;

  # read database entries
  my %databases;
  foreach my $database (@{$conf{'all'}{'databases'}}) {
    $databases{$database}{'class'} = &read_val("",$rawconf{$database.'_class'});
  }
  %{$conf{'databases'}} = %databases;

  return %conf;
}

# 0 for success, 1 for error
sub change_dir {
  my ($dir,$fake) = @_;

  Bkp::dprint("changing to directory '$dir'\n");
  if ($fake) {
    return 0;
  } else {
    return !chdir($dir);
  }
}

# 0 for success, 1 for failure
sub create_dir {
  my ($dir,$fake) = @_;

  Bkp::dprint("creating directory '$dir'\n");
  if ($fake) {
    return 0;
  } else { 
    return system("mkdir -p '$dir'");
  }
}

sub expire_dir {
  my ($dir,$maxdays,$fake) = @_;
  my @goners = split(/\n/, eval("`find $dir -mtime +$maxdays -maxdepth 1`"));
  while (my $goner = shift(@goners)) {
    my $op = "erasing $goner ..";
    if ($fake or (unlink($goner) == 1)) {
      dprint("$op OK\n");
    } else {
      print("$op ERROR\n");
    }
  }
}

